=== modified file 'NEWS'
--- NEWS	2008-07-04 03:41:21 +0000
+++ NEWS	2008-07-04 03:56:20 +0000
@@ -37,6 +37,10 @@
 
   BUGFIXES:
 
+    * ``check`` handles the split-up .bzr layout correctly, so no longer
+      requires a branch to be present.
+      (Daniel Watkins, #64783)
+
     * Clearer message about how to set the PYTHONPATH if bzrlib can't be
       loaded. 
       (Martin Pool, #205230)
@@ -96,6 +100,10 @@
       parameter.  If you have a subclass of ``Branch`` that overrides
       ``pull`` then you should add this parameter.  (Andrew Bennetts)
 
+    * ``bzrlib.check.check()`` has been deprecated in favour of the more
+      aptly-named ``bzrlib.check.check_branch()``.
+      (Daniel Watkins)
+
     * ``Tree.print_file`` and ``Repository.print_file`` are deprecated.
       These methods are bad APIs because they write directly to sys.stdout.
       bzrlib does not use them internally, and there are no direct tests
@@ -106,6 +114,11 @@
     * ``cat`` command no longer uses ``Tree.print_file()`` internally.
       (Alexander Belchenko)
 
+    * New class method ``BzrDir.open_containing_tree_branch_or_repository``
+      which eases the discovery of the tree, the branch and the repository
+      containing a given location.
+      (Daniel Watkins)
+
     * New ``versionedfile.KeyMapper`` interface to abstract out the access to
       underlying .knit/.kndx etc files in repositories with partitioned
       storage. (Robert Collins)

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2008-07-02 01:40:02 +0000
+++ bzrlib/builtins.py	2008-07-04 03:29:14 +0000
@@ -2385,12 +2385,14 @@
 
 
 class cmd_check(Command):
-    """Validate consistency of branch history.
-
-    This command checks various invariants about the branch storage to
-    detect data corruption or bzr bugs.
-
-    Output fields:
+    """Validate working tree structure, branch consistency and repository
+    history.
+
+    This command checks various invariants about branch and repository storage
+    to detect data corruption or bzr bugs.
+
+    The working tree and branch checks will only give output if a problem is
+    detected. The output fields of the repository check are:
 
         revisions: This is just the number of revisions checked.  It doesn't
             indicate a problem.
@@ -2408,42 +2410,14 @@
     """
 
     _see_also = ['reconcile']
-    takes_args = ['branch?']
+    takes_args = ['path?']
     takes_options = ['verbose']
 
-    def run(self, branch=None, verbose=False):
-        from bzrlib.check import check
-        if branch is None:
-            branch_obj = Branch.open_containing('.')[0]
-        else:
-            branch_obj = Branch.open(branch)
-        check(branch_obj, verbose)
-        # bit hacky, check the tree parent is accurate
-        try:
-            if branch is None:
-                tree = WorkingTree.open_containing('.')[0]
-            else:
-                tree = WorkingTree.open(branch)
-        except (errors.NoWorkingTree, errors.NotLocalUrl):
-            pass
-        else:
-            # This is a primitive 'check' for tree state. Currently this is not
-            # integrated into the main check logic as yet.
-            tree.lock_read()
-            try:
-                tree_basis = tree.basis_tree()
-                tree_basis.lock_read()
-                try:
-                    repo_basis = tree.branch.repository.revision_tree(
-                        tree.last_revision())
-                    if len(list(repo_basis.iter_changes(tree_basis))):
-                        raise errors.BzrCheckError(
-                            "Mismatched basis inventory content.")
-                    tree._validate()
-                finally:
-                    tree_basis.unlock()
-            finally:
-                tree.unlock()
+    def run(self, path=None, verbose=False):
+        from bzrlib.check import check_dwim
+        if path is None:
+            path = '.'
+        check_dwim(path, verbose)
 
 
 class cmd_upgrade(Command):

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2008-06-18 07:56:09 +0000
+++ bzrlib/bzrdir.py	2008-07-03 07:06:24 +0000
@@ -61,6 +61,7 @@
     sha_strings,
     sha_string,
     )
+from bzrlib.repository import Repository
 from bzrlib.smart.client import _SmartClient
 from bzrlib.smart import protocol
 from bzrlib.store.versioned import WeaveStore
@@ -844,6 +845,31 @@
         tree, branch = bzrdir._get_tree_branch()
         return tree, branch, relpath
 
+    @classmethod
+    def open_containing_tree_branch_or_repository(klass, location):
+        """Return the working tree, branch and repo contained by a location.
+
+        Returns (tree, branch, repository, relpath).
+        If there is no tree containing the location, tree will be None.
+        If there is no branch containing the location, branch will be None.
+        If there is no repository containing the location, repository will be
+        None.
+        relpath is the portion of the path that is contained by the innermost
+        BzrDir.
+
+        If no tree, branch or repository is found, a NotBranchError is raised.
+        """
+        bzrdir, relpath = klass.open_containing(location)
+        try:
+            tree, branch = bzrdir._get_tree_branch()
+        except errors.NotBranchError:
+            try:
+                repo = bzrdir.find_repository()
+                return None, None, repo, relpath
+            except (errors.NoRepositoryPresent):
+                raise errors.NotBranchError(location)
+        return tree, branch, branch.repository, relpath
+
     def open_repository(self, _unsupported=False):
         """Open the repository object at this BzrDir if one is present.
 

=== modified file 'bzrlib/check.py'
--- bzrlib/check.py	2008-06-11 07:22:00 +0000
+++ bzrlib/check.py	2008-07-03 07:06:24 +0000
@@ -32,12 +32,17 @@
 # raising them.  If there's more than one exception it'd be good to see them
 # all.
 
-from bzrlib import errors
+from bzrlib import errors, osutils
 from bzrlib import repository as _mod_repository
 from bzrlib import revision
+from bzrlib.branch import Branch
+from bzrlib.bzrdir import BzrDir
 from bzrlib.errors import BzrCheckError
+from bzrlib.repository import Repository
+from bzrlib.symbol_versioning import deprecated_function, deprecated_in
+from bzrlib.trace import log_error, note
 import bzrlib.ui
-from bzrlib.trace import log_error, note
+from bzrlib.workingtree import WorkingTree
 
 class Check(object):
     """Check a repository"""
@@ -236,21 +241,61 @@
             seen_names[path] = True
 
 
+@deprecated_function(deprecated_in((1,6,0)))
 def check(branch, verbose):
     """Run consistency checks on a branch.
     
     Results are reported through logging.
     
+    Deprecated in 1.6.  Please use check_branch instead.
+
+    :raise BzrCheckError: if there's a consistency error.
+    """
+    check_branch(branch, verbose)
+
+
+def check_branch(branch, verbose):
+    """Run consistency checks on a branch.
+
+    Results are reported through logging.
+
     :raise BzrCheckError: if there's a consistency error.
     """
     branch.lock_read()
     try:
         branch_result = branch.check()
-        repo_result = branch.repository.check([branch.last_revision()])
     finally:
         branch.unlock()
     branch_result.report_results(verbose)
-    repo_result.report_results(verbose)
-
-
-
+
+
+def check_dwim(path, verbose):
+    tree, branch, repo, relpath = BzrDir.open_containing_tree_branch_or_repository(path)
+
+    if tree is not None:
+        note("Checking working tree at '%s'." 
+             % (tree.bzrdir.root_transport.base,))
+        tree._check()
+
+    if branch is not None:
+        # We have a branch
+        if repo is None:
+            # The branch is in a shared repository
+            repo = branch.repository
+        branches = [branch]
+    elif repo is not None:
+        branches = repo.find_branches(using=True)
+
+    if repo is not None:
+        repo.lock_read()
+        try:
+            note("Checking repository at '%s'."
+                 % (repo.bzrdir.root_transport.base,))
+            result = repo.check()
+            result.report_results(verbose)
+            for branch in branches:
+                note("Checking branch at '%s'."
+                     % (branch.bzrdir.root_transport.base,))
+                check_branch(branch, verbose)
+        finally:
+            repo.unlock()

=== modified file 'bzrlib/tests/blackbox/test_check.py'
--- bzrlib/tests/blackbox/test_check.py	2007-10-24 20:11:21 +0000
+++ bzrlib/tests/blackbox/test_check.py	2008-03-10 23:45:32 +0000
@@ -32,4 +32,14 @@
     def test_check_one_commit_tree(self):
         tree = self.make_branch_and_tree('.')
         tree.commit('hallelujah')
-        self.run_bzr('check')
+        out, err = self.run_bzr('check')
+        self.assertContainsRe(err, r"^Checking working tree at '.*'\.\n"
+                                   r"Checking repository at '.*'\.\n"
+                                   r"checked repository.*\n"
+                                   r"     1 revisions\n"
+                                   r"     0 file-ids\n"
+                                   r"     0 unique file texts\n"
+                                   r"     0 repeated file texts\n"
+                                   r"     0 unreferenced text versions\n"
+                                   r"Checking branch at '.*'\.\n"
+                                   r"checked branch.*\n$")

=== modified file 'bzrlib/tests/bzrdir_implementations/test_bzrdir.py'
--- bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2008-06-11 04:20:16 +0000
+++ bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2008-07-02 08:57:24 +0000
@@ -36,7 +36,7 @@
     workingtree,
     )
 from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
-from bzrlib.check import check
+from bzrlib.check import check_branch
 from bzrlib.errors import (FileExists,
                            NoSuchRevision,
                            NoSuchFile,
@@ -1484,7 +1484,8 @@
             finally:
                 pb.finished()
             # and it should pass 'check' now.
-            check(bzrdir.BzrDir.open(self.get_url('.')).open_branch(), False)
+            check_branch(bzrdir.BzrDir.open(self.get_url('.')).open_branch(),
+                         False)
 
     def test_format_description(self):
         dir = self.make_bzrdir('.')

=== modified file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	2008-05-02 07:31:24 +0000
+++ bzrlib/tests/test_bzrdir.py	2008-07-02 09:33:19 +0000
@@ -19,6 +19,7 @@
 For interface contract tests, see tests/bzr_dir_implementations.
 """
 
+import os
 import os.path
 from StringIO import StringIO
 import subprocess
@@ -436,6 +437,9 @@
         if not self.vfs_transport_factory == MemoryServer:
             self.transport_readonly_server = HttpServer
 
+    def local_branch_path(self, branch):
+         return os.path.realpath(urlutils.local_path_from_url(branch.base))
+
     def test_open_containing(self):
         self.assertRaises(NotBranchError, bzrdir.BzrDir.open_containing,
                           self.get_readonly_url(''))
@@ -447,6 +451,93 @@
         branch, relpath = bzrdir.BzrDir.open_containing(self.get_readonly_url('g/p/q'))
         self.assertEqual('g/p/q', relpath)
 
+    def test_open_containing_tree_branch_or_repository_empty(self):
+        self.assertRaises(errors.NotBranchError,
+            bzrdir.BzrDir.open_containing_tree_branch_or_repository,
+            self.get_readonly_url(''))
+
+    def test_open_containing_tree_branch_or_repository_all(self):
+        self.make_branch_and_tree('topdir')
+        tree, branch, repo, relpath = \
+            bzrdir.BzrDir.open_containing_tree_branch_or_repository(
+                'topdir/foo')
+        self.assertEqual(os.path.realpath('topdir'),
+                         os.path.realpath(tree.basedir))
+        self.assertEqual(os.path.realpath('topdir'),
+                         self.local_branch_path(branch))
+        self.assertEqual(
+            os.path.realpath(os.path.join('topdir', '.bzr', 'repository')),
+            repo.bzrdir.transport.local_abspath('repository'))
+        self.assertEqual(relpath, 'foo')
+
+    def test_open_containing_tree_branch_or_repository_no_tree(self):
+        self.make_branch('branch')
+        tree, branch, repo, relpath = \
+            bzrdir.BzrDir.open_containing_tree_branch_or_repository(
+                'branch/foo')
+        self.assertEqual(tree, None)
+        self.assertEqual(os.path.realpath('branch'),
+                         self.local_branch_path(branch))
+        self.assertEqual(
+            os.path.realpath(os.path.join('branch', '.bzr', 'repository')),
+            repo.bzrdir.transport.local_abspath('repository'))
+        self.assertEqual(relpath, 'foo')
+
+    def test_open_containing_tree_branch_or_repository_repo(self):
+        self.make_repository('repo')
+        tree, branch, repo, relpath = \
+            bzrdir.BzrDir.open_containing_tree_branch_or_repository(
+                'repo')
+        self.assertEqual(tree, None)
+        self.assertEqual(branch, None)
+        self.assertEqual(
+            os.path.realpath(os.path.join('repo', '.bzr', 'repository')),
+            repo.bzrdir.transport.local_abspath('repository'))
+        self.assertEqual(relpath, '')
+
+    def test_open_containing_tree_branch_or_repository_shared_repo(self):
+        self.make_repository('shared', shared=True)
+        bzrdir.BzrDir.create_branch_convenience('shared/branch',
+                                                force_new_tree=False)
+        tree, branch, repo, relpath = \
+            bzrdir.BzrDir.open_containing_tree_branch_or_repository(
+                'shared/branch')
+        self.assertEqual(tree, None)
+        self.assertEqual(os.path.realpath('shared/branch'),
+                         self.local_branch_path(branch))
+        self.assertEqual(
+            os.path.realpath(os.path.join('shared', '.bzr', 'repository')),
+            repo.bzrdir.transport.local_abspath('repository'))
+        self.assertEqual(relpath, '')
+
+    def test_open_containing_tree_branch_or_repository_branch_subdir(self):
+        self.make_branch_and_tree('foo')
+        self.build_tree(['foo/bar/'])
+        tree, branch, repo, relpath = \
+            bzrdir.BzrDir.open_containing_tree_branch_or_repository(
+                'foo/bar')
+        self.assertEqual(os.path.realpath('foo'),
+                         os.path.realpath(tree.basedir))
+        self.assertEqual(os.path.realpath('foo'),
+                         self.local_branch_path(branch))
+        self.assertEqual(
+            os.path.realpath(os.path.join('foo', '.bzr', 'repository')),
+            repo.bzrdir.transport.local_abspath('repository'))
+        self.assertEqual(relpath, 'bar')
+
+    def test_open_containing_tree_branch_or_repository_repo_subdir(self):
+        self.make_repository('bar')
+        self.build_tree(['bar/baz/'])
+        tree, branch, repo, relpath = \
+            bzrdir.BzrDir.open_containing_tree_branch_or_repository(
+                'bar/baz')
+        self.assertEqual(tree, None)
+        self.assertEqual(branch, None)
+        self.assertEqual(
+            os.path.realpath(os.path.join('bar', '.bzr', 'repository')),
+            repo.bzrdir.transport.local_abspath('repository'))
+        self.assertEqual(relpath, 'baz')
+
     def test_open_containing_from_transport(self):
         self.assertRaises(NotBranchError, bzrdir.BzrDir.open_containing_from_transport,
                           get_transport(self.get_readonly_url('')))
@@ -461,17 +552,13 @@
         self.assertEqual('g/p/q', relpath)
 
     def test_open_containing_tree_or_branch(self):
-        def local_branch_path(branch):
-             return os.path.realpath(
-                urlutils.local_path_from_url(branch.base))
-
         self.make_branch_and_tree('topdir')
         tree, branch, relpath = bzrdir.BzrDir.open_containing_tree_or_branch(
             'topdir/foo')
         self.assertEqual(os.path.realpath('topdir'),
                          os.path.realpath(tree.basedir))
         self.assertEqual(os.path.realpath('topdir'),
-                         local_branch_path(branch))
+                         self.local_branch_path(branch))
         self.assertIs(tree.bzrdir, branch.bzrdir)
         self.assertEqual('foo', relpath)
         # opening from non-local should not return the tree
@@ -485,20 +572,16 @@
             'topdir/foo')
         self.assertIs(tree, None)
         self.assertEqual(os.path.realpath('topdir/foo'),
-                         local_branch_path(branch))
+                         self.local_branch_path(branch))
         self.assertEqual('', relpath)
 
     def test_open_tree_or_branch(self):
-        def local_branch_path(branch):
-             return os.path.realpath(
-                urlutils.local_path_from_url(branch.base))
-
         self.make_branch_and_tree('topdir')
         tree, branch = bzrdir.BzrDir.open_tree_or_branch('topdir')
         self.assertEqual(os.path.realpath('topdir'),
                          os.path.realpath(tree.basedir))
         self.assertEqual(os.path.realpath('topdir'),
-                         local_branch_path(branch))
+                         self.local_branch_path(branch))
         self.assertIs(tree.bzrdir, branch.bzrdir)
         # opening from non-local should not return the tree
         tree, branch = bzrdir.BzrDir.open_tree_or_branch(
@@ -509,7 +592,7 @@
         tree, branch = bzrdir.BzrDir.open_tree_or_branch('topdir/foo')
         self.assertIs(tree, None)
         self.assertEqual(os.path.realpath('topdir/foo'),
-                         local_branch_path(branch))
+                         self.local_branch_path(branch))
 
     def test_open_from_transport(self):
         # transport pointing at bzrdir should give a bzrdir with root transport

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2008-07-02 01:40:02 +0000
+++ bzrlib/workingtree.py	2008-07-04 03:29:14 +0000
@@ -2453,6 +2453,20 @@
         self.set_conflicts(un_resolved)
         return un_resolved, resolved
 
+    @needs_read_lock
+    def _check(self):
+        tree_basis = self.basis_tree()
+        tree_basis.lock_read()
+        try:
+            repo_basis = self.branch.repository.revision_tree(
+                self.last_revision())
+            if len(list(repo_basis.iter_changes(tree_basis))) > 0:
+                raise errors.BzrCheckError(
+                    "Mismatched basis inventory content.")
+            self._validate()
+        finally:
+            tree_basis.unlock()
+
     def _validate(self):
         """Validate internal structures.
 

